@@ -1,21 +1,9 @@
 # Copyright 2008-2015 Free Software Foundation, Inc.
 # This file is part of GNU Radio
 #
-# GNU Radio Companion is free software; you can redistribute it and/or
-# modify it under the terms of the GNU General Public License
-# as published by the Free Software Foundation; either version 2
-# of the License, or (at your option) any later version.
+# SPDX-License-Identifier: GPL-2.0-or-later
 #
-# GNU Radio Companion is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 
-from __future__ import absolute_import, print_function
 
 import collections
 import itertools
@@ -56,6 +44,7 @@ class FlowGraph(Element):
 
         self._eval_cache = {}
         self.namespace = {}
+        self.imported_names = []
 
         self.grc_file_path = ''
 
@@ -79,7 +68,8 @@ class FlowGraph(Element):
         Returns:
             a sorted list of variable blocks in order of dependency (indep -> dep)
         """
-        variables = [block for block in self.iter_enabled_blocks() if block.is_variable]
+        variables = [block for block in self.iter_enabled_blocks()
+                     if block.is_variable]
         return expr_utils.sort_objects(variables, attrgetter('name'), methodcaller('get_var_make'))
 
     def get_parameters(self):
@@ -89,14 +79,57 @@ class FlowGraph(Element):
         Returns:
             a list of parameterized variables
         """
-        parameters = [b for b in self.iter_enabled_blocks() if b.key == 'parameter']
+        parameters = [b for b in self.iter_enabled_blocks()
+                      if b.key == 'parameter']
         return parameters
 
+    def get_snippets(self):
+        """
+        Get a set of all code snippets (Python) in this flow graph namespace.
+
+        Returns:
+            a list of code snippets
+        """
+        return [b for b in self.iter_enabled_blocks() if b.key == 'snippet']
+
+    def get_snippets_dict(self, section=None):
+        """
+        Get a dictionary of code snippet information for a particular section.
+
+        Args:
+            section: string specifier of section of snippets to return, section=None returns all
+
+        Returns:
+            a list of code snippets dicts
+        """
+        snippets = self.get_snippets()
+        if not snippets:
+            return []
+
+        output = []
+        for snip in snippets:
+            d = {}
+            sect = snip.params['section'].value
+            d['section'] = sect
+            d['priority'] = snip.params['priority'].value
+            d['lines'] = snip.params['code'].value.splitlines()
+            d['def'] = 'def snipfcn_{}(self):'.format(snip.name)
+            d['call'] = 'snipfcn_{}(tb)'.format(snip.name)
+            if not section or sect == section:
+                output.append(d)
+
+        # Sort by descending priority
+        if section:
+            output = sorted(output, key=lambda x: x['priority'], reverse=True)
+
+        return output
+
     def get_monitors(self):
         """
         Get a list of all ControlPort monitors
         """
-        monitors = [b for b in self.iter_enabled_blocks() if 'ctrlport_monitor' in b.key]
+        monitors = [b for b in self.iter_enabled_blocks()
+                    if 'ctrlport_monitor' in b.key]
         return monitors
 
     def get_python_modules(self):
@@ -159,7 +192,18 @@ class FlowGraph(Element):
                 filename=shlex.quote(file_path))
             return shlex.split(run_command) if split else run_command
         except Exception as e:
-            raise ValueError("Can't parse run command {!r}: {}".format(run_command, e))
+            raise ValueError(
+                "Can't parse run command {!r}: {}".format(run_command, e))
+
+    def get_imported_names(self):
+        """
+        Get a lis of imported names.
+        These names may not be used as id's
+
+        Returns:
+            a list of imported names
+        """
+        return self.imported_names
 
     ##############################################
     # Access Elements
@@ -176,7 +220,7 @@ class FlowGraph(Element):
         return elements
 
     def children(self):
-        return itertools.chain(self.iter_enabled_blocks(), self.connections)
+        return itertools.chain(self.blocks, self.connections)
 
     def rewrite(self):
         """
@@ -187,6 +231,10 @@ class FlowGraph(Element):
 
     def renew_namespace(self):
         namespace = {}
+        # Before renewing the namespace, clear it
+        # to get rid of entries of blocks that
+        # are no longer valid ( deleted, disabled, ...)
+        self.namespace.clear()
         # Load imports
         for expr in self.imports():
             try:
@@ -197,44 +245,55 @@ class FlowGraph(Element):
                 # this is ok behavior, unfortunately we could be hiding other import bugs
                 pass
             except Exception:
-                log.exception('Failed to evaluate import expression "{0}"'.format(expr), exc_info=True)
+                log.exception('Failed to evaluate import expression "{0}"'.format(
+                    expr), exc_info=True)
                 pass
 
+        self.imported_names = list(namespace.keys())
+
         for id, expr in self.get_python_modules():
             try:
                 module = types.ModuleType(id)
                 exec(expr, module.__dict__)
                 namespace[id] = module
             except Exception:
-                log.exception('Failed to evaluate expression in module {0}'.format(id), exc_info=True)
+                log.exception(
+                    'Failed to evaluate expression in module {0}'.format(id), exc_info=True)
                 pass
 
         # Load parameters
         np = {}  # params don't know each other
         for parameter_block in self.get_parameters():
             try:
-                value = eval(parameter_block.params['value'].to_code(), namespace)
+                value = eval(
+                    parameter_block.params['value'].to_code(), namespace)
                 np[parameter_block.name] = value
             except Exception:
-                log.exception('Failed to evaluate parameter block {0}'.format(parameter_block.name), exc_info=True)
+                log.exception('Failed to evaluate parameter block {0}'.format(
+                    parameter_block.name), exc_info=True)
                 pass
         namespace.update(np)  # Merge param namespace
 
+        # We need the updated namespace to evaluate the variable blocks
+        # otherwise sometimes variable_block rewrite / eval fails
+        self.namespace.update(namespace)
         # Load variables
         for variable_block in self.get_variables():
             try:
                 variable_block.rewrite()
-                value = eval(variable_block.value, namespace, variable_block.namespace)
+                value = eval(variable_block.value, namespace,
+                             variable_block.namespace)
                 namespace[variable_block.name] = value
-            except TypeError: #Type Errors may happen, but that desn't matter as they are displayed in the gui
+                # rewrite on subsequent blocks depends on an updated self.namespace
+                self.namespace.update(namespace)
+            except TypeError:  # Type Errors may happen, but that doesn't matter as they are displayed in the gui
                 pass
             except Exception:
-                log.exception('Failed to evaluate variable block {0}'.format(variable_block.name), exc_info=True)
+                log.exception('Failed to evaluate variable block {0}'.format(
+                    variable_block.name), exc_info=True)
                 pass
 
-        self.namespace.clear()
         self._eval_cache.clear()
-        self.namespace.update(namespace)
 
     def evaluate(self, expr, namespace=None, local_namespace=None):
         """
@@ -287,7 +346,7 @@ class FlowGraph(Element):
         connection = self.parent_platform.Connection(
             parent=self, source=porta, sink=portb)
         self.connections.add(connection)
-            
+
         return connection
 
     def disconnect(self, *ports):
@@ -336,7 +395,10 @@ class FlowGraph(Element):
         data['blocks'] = [b.export_data() for b in sorted(self.blocks, key=block_order)
                           if b is not self.options_block]
         data['connections'] = sorted(c.export_data() for c in self.connections)
-        data['metadata'] = {'file_format': FLOW_GRAPH_FILE_FORMAT_VERSION}
+        data['metadata'] = {
+            'file_format': FLOW_GRAPH_FILE_FORMAT_VERSION,
+            'grc_version': self.parent_platform.config.version
+        }
         return data
 
     def _build_depending_hier_block(self, block_id):
@@ -349,7 +411,8 @@ class FlowGraph(Element):
             cwd=self.grc_file_path
         )
         if file_path:  # grc file found. load and get block
-            self.parent_platform.load_and_generate_flow_graph(file_path, hier_only=True)
+            self.parent_platform.load_and_generate_flow_graph(
+                file_path, hier_only=True)
             return self.new_block(block_id)  # can be None
 
     def import_data(self, data):
@@ -376,13 +439,13 @@ class FlowGraph(Element):
             block = (
                 self.new_block(block_id) or
                 self._build_depending_hier_block(block_id) or
-                self.new_block(block_id='_dummy', missing_block_id=block_id, **block_data)
+                self.new_block(block_id='_dummy',
+                               missing_block_id=block_id, **block_data)
             )
 
             block.import_data(**block_data)
 
-        self.rewrite()  # TODO: Figure out why this has to be called twice to populate bus ports correctly
-        self.rewrite()  # evaluate stuff like nports before adding connections
+        self.rewrite()
 
         # build the connections
         def verify_and_get_port(key, block, dir):
@@ -396,15 +459,16 @@ class FlowGraph(Element):
                 if block.is_dummy_block:
                     port = block.add_missing_port(key, dir)
                 else:
-                    raise LookupError('%s key %r not in %s block keys' % (dir, key, dir))
+                    raise LookupError(
+                        '%s key %r not in %s block keys' % (dir, key, dir))
             return port
 
         had_connect_errors = False
         _blocks = {block.name: block for block in self.blocks}
 
-        try:
-            # TODO: Add better error handling if no connections exist in the flowgraph file.
-            for src_blk_id, src_port_id, snk_blk_id, snk_port_id in data.get('connections', []):
+        # TODO: Add better error handling if no connections exist in the flowgraph file.
+        for src_blk_id, src_port_id, snk_blk_id, snk_port_id in data.get('connections', []):
+            try:
                 source_block = _blocks[src_blk_id]
                 sink_block = _blocks[snk_blk_id]
 
@@ -414,24 +478,27 @@ class FlowGraph(Element):
                         src_port_id, snk_port_id, source_block, sink_block)
 
                 # build the connection
-                source_port = verify_and_get_port(src_port_id, source_block, 'source')
-                sink_port = verify_and_get_port(snk_port_id, sink_block, 'sink')
+                source_port = verify_and_get_port(
+                    src_port_id, source_block, 'source')
+                sink_port = verify_and_get_port(
+                    snk_port_id, sink_block, 'sink')
 
                 self.connect(source_port, sink_port)
 
-        except (KeyError, LookupError) as e:
-            Messages.send_error_load(
-                'Connection between {}({}) and {}({}) could not be made.\n\t{}'.format(
-                    src_blk_id, src_port_id, snk_blk_id, snk_port_id, e))
-            had_connect_errors = True
+            except (KeyError, LookupError) as e:
+                Messages.send_error_load(
+                    'Connection between {}({}) and {}({}) could not be made.\n\t{}'.format(
+                        src_blk_id, src_port_id, snk_blk_id, snk_port_id, e))
+                had_connect_errors = True
 
         for block in self.blocks:
-            if block.is_dummy_block :
+            if block.is_dummy_block:
                 block.rewrite()      # Make ports visible
                 # Flowgraph errors depending on disabled blocks are not displayed
                 # in the error dialog box
                 # So put a message into the Property window of the dummy block
-                block.add_error_message('Block id "{}" not found.'.format(block.key))
+                block.add_error_message(
+                    'Block id "{}" not found.'.format(block.key))
 
         self.rewrite()  # global rewrite
         return had_connect_errors
